function Pipeline(...middlewares) {
  const stack = middlewares;

  const push = (...middlewares) => {
    stack.push(...middlewares);
  };

  const execute = function (context) {
    try {
      let prevIndex = -1;

      const runner = function (index) {
        try {
          if (index === prevIndex) {
            throw new Error('next() called multiple times');
          }

          prevIndex = index;
          const middleware = stack[index];

          const _temp = function () {
            if (middleware) {
              return Promise.resolve(middleware(context, runner.bind(null, index + 1))).then(function () {});
            }
          }();

          return Promise.resolve(_temp && _temp.then ? _temp.then(function () {}) : void 0);
        } catch (e) {
          return Promise.reject(e);
        }
      };

      return Promise.resolve(runner(0)).then(function () {});
    } catch (e) {
      return Promise.reject(e);
    }
  };

  return {
    push,
    execute
  };
}

exports.Pipeline = Pipeline;
//# sourceMappingURL=index.js.map
